package adminkindboxreqhandler

import (
	"net/http"
	"strconv"

	params "git.gocasts.ir/ebhomengo/niki/param"
	param "git.gocasts.ir/ebhomengo/niki/param/admin/kind_box_req"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	httpmsg "git.gocasts.ir/ebhomengo/niki/pkg/http_msg"
	"github.com/labstack/echo/v4"
)

// Accept godoc
// @Summary      Accept kind box request by admin
// @Tags         KindBoxReq
// @Accept       json
// @Produce      json
// @Param 		 id path int true "KindBoxReq ID"
// @Param 		 Request body param.KindBoxReqAcceptRequest true "Accept KindBoxReq Request Body"
// @Success      200  {object}  param.KindBoxReqAcceptResponse
// @Failure      400  {string}  "Bad request"
// @Security 	 AuthBearerAdmin
// @Router       /admin/kindboxreqs/accept-kind-box-req/{id} [patch].
func (h Handler) Accept(c echo.Context) error {
	var req param.KindBoxReqAcceptRequest
	if bErr := c.Bind(&req); bErr != nil {
		return echo.NewHTTPError(http.StatusBadRequest)
	}
	num, cErr := strconv.ParseUint(c.Param("id"), 10, 64)
	if cErr != nil {
		return c.JSON(http.StatusBadRequest, errmsg.ErrorMsgInvalidInput)
	}
	req.ID = uint(num)
	if fieldErrors, err := h.adminKindBoxReqVld.ValidateAcceptRequest(req); err != nil {
		msg, code := httpmsg.Error(err)

		return c.JSON(code, echo.Map{
			"message": msg,
			"errors":  fieldErrors,
		})
	}

	resp, sErr := h.adminKindBoxReqSvc.Accept(c.Request().Context(), req)
	if sErr != nil {
		msg, code := httpmsg.Error(sErr)

		return echo.NewHTTPError(code, msg)
	}
	go h.notificationSvc.KindBoxReqAccepted(params.NotificationKindBoxReqAccepted{KindBoxReqID: req.ID})
	return c.JSON(http.StatusOK, resp)
}
