package benefactoraddresshandler

import (
	"net/http"

	param "git.gocasts.ir/ebhomengo/niki/param/benefactor/address"
	"git.gocasts.ir/ebhomengo/niki/pkg/claim"
	httpmsg "git.gocasts.ir/ebhomengo/niki/pkg/http_msg"
	"github.com/labstack/echo/v4"
)

// AddAddress godoc
// @Summary      Add a new address for a benefactor
// @Description This endpoint allows an authenticated benefactor to add a new address to their account.
// @Tags         Address
// @Accept       json
// @Produce      json
// @Param        Request body   param.BenefactorAddAddressRequest true "New address details"
// @Success      201  {object}  param.BenefactorAddAddressResponse
// @Failure      400  {string}  "Bad request"
// @Security AuthBearerBenefactor
// @Router       /address/ [post].
func (h Handler) AddAddress(c echo.Context) error {
	req := param.BenefactorAddAddressRequest{}
	if bErr := c.Bind(&req); bErr != nil {
		return echo.NewHTTPError(http.StatusBadRequest)
	}
	claims := claim.GetClaimsFromEchoContext(c)
	req.BenefactorID = claims.UserID

	if fieldErrors, err := h.addressVld.ValidateAddAddress(req); err != nil {
		msg, code := httpmsg.Error(err)

		return c.JSON(code, echo.Map{
			"message": msg,
			"errors":  fieldErrors,
		})
	}
	resp, sErr := h.addressSvc.Add(c.Request().Context(), req)

	if sErr != nil {
		msg, code := httpmsg.Error(sErr)

		return echo.NewHTTPError(code, msg)
	}

	return c.JSON(http.StatusCreated, resp)
}
