package mysqladdress

import (
	"context"
	"time"

	"git.gocasts.ir/ebhomengo/niki/entity"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	richerror "git.gocasts.ir/ebhomengo/niki/pkg/rich_error"
	"git.gocasts.ir/ebhomengo/niki/repository/mysql"
)

func (d *DB) GetAllProvinces(ctx context.Context) ([]entity.Province, error) {
	const op = "mysqladdress.GetAllProvinces"

	provinces := make([]entity.Province, 0)

	rows, err := d.conn.Conn().QueryContext(ctx, `SELECT * FROM provinces;`)
	if err != nil {
		return nil, richerror.New(op).WithErr(err).
			WithMessage(errmsg.ErrorMsgSomethingWentWrong).WithKind(richerror.KindUnexpected)
	}

	defer rows.Close()

	for rows.Next() {
		province, err := scanProvince(rows)
		if err != nil {
			return nil, richerror.New(op).WithErr(err).
				WithMessage(errmsg.ErrorMsgSomethingWentWrong).WithKind(richerror.KindUnexpected)
		}

		provinces = append(provinces, province)
	}

	if err := rows.Err(); err != nil {
		return nil, richerror.New(op).WithErr(err).
			WithMessage(errmsg.ErrorMsgSomethingWentWrong).WithKind(richerror.KindUnexpected)
	}

	return provinces, nil
}

func scanProvince(scanner mysql.Scanner) (entity.Province, error) {
	var createdAt, updatedAt time.Time
	var province entity.Province

	err := scanner.Scan(&province.ID, &province.Name, &createdAt, &updatedAt)

	return province, err
}
